<?php defined('SYSPATH') OR die('No direct access allowed.');
/**
 * Traps controller
 *
 *  op5, and the op5 logo are trademarks, servicemarks, registered servicemarks
 *  or registered trademarks of op5 AB.
 *  All other trademarks, servicemarks, registered trademarks, and registered
 *  servicemarks mentioned herein may be the property of their respective owner(s).
 *  The information contained herein is provided AS IS with NO WARRANTY OF ANY
 *  KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY, AND FITNESS FOR A
 *  PARTICULAR PURPOSE.
 */
class Traps_Controller extends Authenticated_Controller
{
	protected function is_authorized_to_edit_service($host, $service)
	{
		if (Auth::instance()->authorized_for('service_edit_all'))
			return true;
		if (!Auth::instance()->authorized_for('service_edit_contact'))
			return false;

		$pool = new ServicePool_Model();
		$set = $pool->all()->reduce_by('host.name', $host, '=')->reduce_by('description', $service, '=');
		# "but just check if the service can be retrieved - if so, we have permissions!"
		# service_edit_contact + service_view_all. Your move.
		$obj = $set->it(array('contacts'), array(), false, false)->current();
		if (!$obj)
			return false;
		if (in_array(Auth::instance()->get_user()->username, $obj->get_contacts(), true))
			return true;
		return false;
	}

	/**
	 *removes trap-related objects
	 */
	public function delete()
	{
		$type = $this->input->post('type');

		switch($type) {
		case 'trap':
			$host = $this->input->post('host') or json::fail('Missing host parameter');
			$service = $this->input->post('service') or json::fail('Missing service parameter');
			$tag = $this->input->post('tag') or json::fail('Missing tag parameter');
			if (!$this->is_authorized_to_edit_service($host, $service))
				json::fail('You are not authorized to edit this service');

			if (Trap_Model::delete_trap($host, $service, $tag))
				json::ok('Subservice deleted');
			break;
		case 'message':
			if (!Auth::instance()->authorized_for('manage_trapper'))
				json::fail('You are not authorized to manage trapper');

			$id = $this->input->post('id') or json::fail('Missing id parameter');
			if (TrapLog_Model::delete_message($id))
				json::ok('Log entry deleted');
			break;
		case 'handler':
			if (!Auth::instance()->authorized_for('manage_trapper'))
				json::fail('You are not authorized to manage trapper');

			$id = $this->input->post('id') or json::fail('Missing id parameter');
			if (TrapHandler_Model::delete_handler($id))
				json::ok("Handler deleted");
			break;
		case 'module':
			if (!Auth::instance()->authorized_for('manage_trapper'))
				json::fail('You are not authorized to manage trapper');

			$id = $this->input->post('id') or json::fail('Missing id parameter');
			if (TrapModule_Model::delete_module($id))
				json::ok("Module deleted");
			break;
		case 'matcher':
			if (!Auth::instance()->authorized_for('manage_trapper'))
				json::fail('You are not authorized to manage trapper');

			$n = $this->input->post('n') or json::fail('Missing n parameter');
			if (TrapMatcher_Model::delete_matcher($n))
				json::ok("Matcher deleted");
			break;
		default:
			json::fail('Trying to delete an invalid object type: ' . $type);
			break;
		}
		json::fail('Unknown error');
	}

	/**
	 * changes trap's state
	 */
	public function change_state()
	{
		$host = $this->input->post('host');
		$service = $this->input->post('service');
		$tag = $this->input->post('tag');
		$state = $this->input->post('state');

		if (!$this->is_authorized_to_edit_service($host, $service)) {
			json::fail('You are not authorized to edit this service');
		}

		if (Trap_Model::change_state($host, $service, $tag, $state)) {
			json::ok('State changed');
		}
		json::fail('Unknown error');
	}

	/**
	 * generate a fake trap from localhost and submit
	 */
	public function submit_trap()
	{
		if (!Auth::instance()->authorized_for('manage_trapper')) {
			json::fail('You are not authorized to manage trapper');
		}

		$version = $this->input->post('trap-version-selector');
		$args = array();
		if ($version == 'v1') {
			$args[] = escapeshellarg($this->input->post('v1-enterprise-oid'));
			$args[] = escapeshellarg($this->input->post('v1-agent'));
			$args[] = escapeshellarg($this->input->post('v1-generic-trap'));
			$args[] = escapeshellarg($this->input->post('v1-specific-trap'));
			$args[] = escapeshellarg($this->input->post('v1-uptime'));
		} elseif ($version == 'v2c') {
			$args[] = escapeshellarg($this->input->post('v2-uptime'));
			$args[] = escapeshellarg($this->input->post('v2-trap-oid'));
		}
		$oids = $this->input->post('oid');
		$types = $this->input->post('type');
		$value = $this->input->post('value');
		for ($i = 0; $i < count($oids); $i++) {
			$args[] = escapeshellarg($oids[$i]);
			$args[] = escapeshellarg($types[$i]);
			$args[] = escapeshellarg($value[$i]);
		}

		$cmd = 'snmptrap -Le -v '.substr($version, 1).' -c public localhost '.implode(' ', $args).' 2>&1';
		exec($cmd, $output, $return_var);
		if (!$return_var) {
			json::ok("Trap submitted");
		}
		json::fail($output?implode("\n", $output):"Unknown error");
	}

	/**
	 * upload new handler
	 */
	public function upload_handler()
	{
		if (!Auth::instance()->authorized_for('manage_trapper')) {
			return json::fail('You are not authorized to manage trapper');
		}
		$name = $_POST['handler_name'];
		if (!$name) {
			return json::fail("You provided no name, can't install handler");
		}

		if (isset($_FILES['handler_file'])) {
			if ($_FILES['handler_file']['error']) {
				return json::fail('Upload failed');
			}
			$size = $_FILES['handler_file']['size'];
			$input = $_FILES['handler_file']['tmp_name'];
			if ($size > 65536) {
				return json::fail('The uploaded file was bigger than 65kB - that seems bogus. If not a mistake, you can still upload the handle through traped.');
			}
		} else {
			$input = false;
		}

		$id = isset($_POST['handler_id']) ? $_POST['handler_id'] : null;
		if ($id != null && !is_numeric($id))
			return json::fail("Invalid ID specified");
		if (!$id && !$input) {
			return json::fail("No file specified, and wasn't an update");
		}
		if (!$id) {
			$pool = new TrapHandlerPool_Model();
			$set = $pool->all()->reduce_by('name', $name, '=');
			if ($set->one()) {
				return json::fail("A module already exists with that name");
			}
		}

		if (TrapHandler_Model::upload_handler($name, $input, $id)) {
			return json::ok("Handler created");
		} else {
			return json::fail("Handler couldn't be saved");
		}
	}

	/**
	 * upload new module
	 */
	public function upload_module()
	{
		if (!Auth::instance()->authorized_for('manage_trapper')) {
			return json::fail('You are not authorized to manage trapper');
		}
		$name = $_POST['module_name'];
		if (!$name) {
			return json::fail("You provided no name, can't install module");
		}

		if (isset($_FILES['module_file'])) {
			if ($_FILES['module_file']['error']) {
				return json::fail('Upload failed');
			}
			$size = $_FILES['module_file']['size'];
			$input = $_FILES['module_file']['tmp_name'];
			if ($size > 65536) {
				return json::fail('The uploaded file was bigger than 65kB - that seems bogus. If not a mistake, you can still upload the handle through traped.');
			}
		} else {
			$input = false;
		}

		$id = isset($_POST['module_id']) ? $_POST['module_id'] : null;
		if ($id != null && !is_numeric($id))
			return json::fail("Invalid ID specified");
		if (!$id && !$input) {
			return json::fail("No file specified, and wasn't an update");
		}
		if (!$id) {
			$pool = new TrapModulePool_Model();
			$set = $pool->all()->reduce_by('name', $name, '=');
			if ($set->one()) {
				return json::fail("A module already exists with that name");
			}
		}

		if (TrapModule_Model::upload_module($name, $input, $id)) {
			return json::ok("Module created");
		} else {
			return json::fail("Module couldn't be saved");
		}
	}

	/**
	 * upload new matcher
	 */
	public function upload_matcher()
	{
		if (!Auth::instance()->authorized_for('manage_trapper')) {
			return json::fail('You are not authorized to manage trapper');
		}
		if (!$this->input->post('matcher_oid')) {
			return json::fail('No OID supplied');
		}
		if (!$this->input->post('matcher_handler')) {
			return json::fail('No handler supplied');
		}

		if (!preg_match('/^([.*0-9]+|fallback)$/', $this->input->post('matcher_oid'))) {
			return json::fail("Invalid match pattern supplied: must be an oid, optionally with the wildcard character '*', or the string 'fallback'. Was '".$this->input->post('matcher_oid')."'");
		}

		if (TrapMatcher_Model::upload_matcher($this->input->post('matcher_oid'), $this->input->post('matcher_handler'), $this->input->post('matcher_n', null))) {
			return json::ok("Matcher created");
		} else {
			return json::fail("Handler couldn't be saved");
		}
	}

	/**
	 * edit matching order
	 */
	public function move_matcher()
	{
		if (!Auth::instance()->authorized_for('manage_trapper'))
			return json::fail('You are not authorized to manage trapper');
		if (!$this->input->post('n'))
			return json::fail('Found no matcher to move');
		if (!$this->input->post('direction'))
			return json::fail('Found no direction to move matcher');
		if ($this->input->post('direction') !== 'up' && $this->input->post('direction') !== 'down')
			return json::fail('Invalid direction to move matcher');

		if (TrapMatcher_Model::move_matcher($this->input->post('n'), $this->input->post('direction'))) {
			return json::ok("Matcher moved " . $this->input->post('direction'));
		} else {
			return json::fail('Error trying to move matcher');
		}
	}

	/**
	 * download the complete handler - the listview only gives you the top snippet
	 */
	public function download_code($type, $id)
	{
		assert('$type === "handler" || $type === "module"');
		if ($type == 'handler')
			$pool = new TrapHandlerPool_Model();
		else
			$pool = new TrapModulePool_Model();
		$set = $pool->get_by_query('[trap_'.$type.'s] id = '.((int)$id));
		$obj = $set->it(array('script', 'name'), array(), false, false)->current();
		if (!$obj) {
			print "Couldn't find script " + $id;
			exit(1);
		}
		header('Content-Type: text/plain');
		header('Content-Disposition: attachment; filename="'.$obj->get_name().'.lua"');
		print $obj->get_script();
		exit(0);
	}
}
